<?php

/**
 * @file
 * Slack integration module API functions.
 */

/**
 * Send message to the Slack.
 *
 * @param string $message
 *   The message sent to the channel.
 * @param string $channel
 *   The channel in the Slack service to send messages.
 * @param string $username
 *   The bot name displayed in the channel.
 *
 * @return bool|object
 *   Slack response.
 */
function amocrm_widget_slack_integration_send_message($webhook_url, $message, $channel = '', $username = '', $icon_options = array()) {
  if (!$webhook_url) {
    drupal_set_message('Please, enter the webhook value and try again', 'error');
    return FALSE;
  }
  if ($channel) {
    $message_options['channel'] = $channel;
  }
  else {
    $message_options['channel'] = amocrm_widget_slack_integration_get_default_channel();
  }
  if ($username) {
    $message_options['username'] = $username;
  }
  else {
    $message_options['username'] = amocrm_widget_slack_integration_get_default_username();
  }

  if ($icon_options['type'] == 'emoji') {
    if ($icon_options['emoji']) {
      $message_options['icon_emoji'] = $icon_options['emoji'];
    }
    else {
      $message_options['icon_emoji'] = amocrm_widget_slack_integration_get_default_icon_emoji();
    }
  }
  elseif ($icon_options['type'] == 'image') {
    if ($icon_options['icon']) {
      $message_options['icon_url'] = $icon_options['icon'];
    }
    else {
      $message_options['icon_url'] = amocrm_widget_slack_integration_get_default_icon_url();
    }
  }
  $result = _slack_send_message($webhook_url, $message, $message_options);
  return $result;
}

/**
 * Send message to the Slack with more options.
 *
 * @param string $team_name
 *   Your team name in the Slack.
 * @param string $team_token
 *   The token from "Incoming WebHooks" integration in the Slack.
 * @param string $message
 *   The message sent to the channel.
 * @param array $message_options
 *   An associative array, it can contain:
 *     - channel: The channel in the Slack service to send messages
 *     - username: The bot name displayed in the channel
 *     - icon_emoji: The bot icon displayed in the channel
 *     - icon_url: The bot icon displayed in the channel
 *
 * @return object
 *   Can contain:
 *                          success      fail          fail
 *     - data:                ok         No hooks      Invalid channel specified
 *     - status message:      OK         Not found     Server Error
 *     - code:                200        404           500
 *     - error:               -          Not found     Server Error
 */
function _slack_send_message($webhook_url, $message, $message_options = array()) {
  $headers = array(
    'Content-Type' => 'application/x-www-form-urlencoded',
  );
  $message_options['text'] = _slack_process_message($message);
  $sending_data = 'payload=' . drupal_json_encode($message_options);
  $options = array(
    'method' => 'POST',
    'data' => $sending_data,
    'headers' => $headers,
  );
  $sending_url = $webhook_url;
  $result = drupal_http_request($sending_url, $options);
  return $result;
}

/**
 * Replaces links with slack friendly tags. Strips all other html.
 *
 * @param string $message
 *   The message sent to the channel.
 *
 * @return string
 *   Replaces links with slack friendly tags. Strips all other html.
 */
function _slack_process_message($message) {
  $regexp = "<a\s[^>]*href=(\"??)([^\" >]*?)\\1[^>]*>(.*)<\/a>";
  if (preg_match_all("/$regexp/siU", $message, $matches, PREG_SET_ORDER)) {
    $i = 1;
    foreach ($matches as $match) {
      $new_link = "<$match[2] | $match[3]>";
      $links['link-' . $i] = $new_link;
      $message = str_replace($match[0], 'link-' . $i, $message);
      $i++;
      $message = strip_tags($message);
      foreach ($links as $id => $link) {
        $message = str_replace($id, $link, $message);
      }
    }
  }
  return $message;
}

/**
 * Get default Webhook URL.
 *
 * @return string
 *   Get default Webhook URL.
 */
function amocrm_widget_slack_integration_get_default_webhook_url() {
  $channel = variable_get('amocrm_widget_slack_integration_webhook_url');
  return $channel;
}

/**
 * Get default team channel.
 *
 * @return string
 *   Get default team channel.
 */
function amocrm_widget_slack_integration_get_default_channel() {
  $channel = variable_get('amocrm_widget_slack_integration_channel');
  return $channel;
}

/**
 * Get default Slack bot username.
 *
 * @return string
 *   Get default Slack bot username.
 */
function amocrm_widget_slack_integration_get_default_username() {
  $username = variable_get('amocrm_widget_slack_integration_username');
  return $username;
}

/**
 * Get default Slack bot icon.
 *
 * @return string
 *   Get default Slack bot icon.
 */
function amocrm_widget_slack_integration_get_default_icon_url() {
  $icon['url'] = variable_get('amocrm_widget_slack_integration_icon_url');
  return $icon['url'];
}

/**
 * Get default Slack bot icon emoji.
 *
 * @return string
 *   Get default Slack bot icon emoji.
 */
function amocrm_widget_slack_integration_get_default_icon_emoji() {
  $icon['emoji'] = variable_get('amocrm_widget_slack_integration_icon_emoji');
  return $icon['emoji'];
}

/**
 * Get default Slack bot image type radios value.
 *
 * @return string
 *   Get default Slack bot image type radios value.
 */
function amocrm_widget_slack_integration_get_default_icon_type() {
  $select_image = variable_get('amocrm_widget_slack_integration_icon_type', 'none');
  return $select_image;
}

/**
 * Call Slack Web API method.
 */
function amocrm_widget_slack_integration_call_method($method_name, $params) {
  $data = array('token' => variable_get('amocrm_widget_slack_integration_token')) + $params;
  $api_url = variable_get('amocrm_widget_slack_integration_api_url');
  $url = url($api_url . $method_name, array(
    'external' => TRUE,
    'query' => $data,
  ));
  $response = drupal_http_request($url);
  if (!empty($response->data)) {
    return drupal_json_decode($response->data);
  }

  return FALSE;
}

/**
 * Create Slack channel.
 */
function amocrm_widget_slack_integration_create_channel($name) {
  $data = array('name' => $name);
  $channel = amocrm_widget_slack_integration_call_method(SLACK_CREATE_CHANNEL_METHOD, $data);
  if ($channel['ok']) {
    return $channel['channel'];
  }

  return FALSE;
}

/**
 * Archive Slack channel.
 */
function amocrm_widget_slack_integration_archive_channel($channel_id) {
  $data = array('channel' => $channel_id);
  $request = amocrm_widget_slack_integration_call_method(SLACK_ARCHIVE_CHANNEL_METHOD, $data);
  if ($request['ok']) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Post Slack message.
 */
function amocrm_widget_slack_integration_post_message($data) {
  $message = amocrm_widget_slack_integration_call_method(SLACK_POST_MESSAGE_METHOD, $data);
  if ($message['ok']) {
    $message = $message['message'];

    return $message;
  }

  return FALSE;
}

/**
 * Get channel info.
 */
function amocrm_widget_slack_integration_channel_information($channel_id) {
  $data = array('channel' => $channel_id);
  $info = amocrm_widget_slack_integration_call_method(SLACK_CHANNELS_INFO_METHOD, $data);
  if ($info['ok']) {
    return $info['channel'];
  }
  else {
    return FALSE;
  }
}
